theory MPoly_Bootstrap
imports "../poly/MPoly"
begin (*<*)

type_notation (xsymbols)
  poly_mapping ("(_ \<Rightarrow>\<^sub>0 /_)" [22, 21] 21)
(*>*)

text {*
  We present the construction of a type @{typ "'a mpoly"} for multivariate polynomials,
  where @{typ "'a"} is the type of coefficients.

  We choose to represent variables by natural numbers (rather than names) for three reasons.
  First, concrete names are an aspect of presentation rather than representation.
  For example, the polynomials @{text "x\<^sup>2 + 2"} and @{text "y\<^sup>2 + 2"} are considered equivalent.
  Second, the natural numbers provide an infinite supply of variables, whereas the type @{typ "'a mpoly"} ensures that only a finite subset is used.
  Thus, fresh variables are easy to find and no name binding issues arise.
  Third, certain algorithms require a notion of variable order, and this is most
  naturally expressed using natural numbers.

  This choice aligns well with the modern mathematical view of
  polynomials as functions rather than symbolic expressions.
  Hence, @{typ "'a mpoly"} is the type of all functions from power products to coefficients that are zero almost everywhere; and a power product (type @{text "power_product"}) is a function from variables to exponents (i.e., natural numbers) that are zero almost everywhere, too.
  As this construction is very close to how mathematicians define multivariate polynomials, experts from computer algebra are easily convinced that this type in fact models multivariate polynomials.

  In detail, we encapsulate the property of being zero almost everywhere in the type @{typ "'b \<Rightarrow>\<^sub>0 'a"} of almost-everywhere-zero (AEZ) maps.
  This type contains all functions @{term [source] "f :: 'b \<Rightarrow> 'a"} that satisfy @{text "finite {b. f b \<noteq> 0}"}.
  \footnote{%
    The constant @{term "0"} is overloaded in Isabelle,
    which imposes a type class constraint @{class zero} on the result type @{typ 'a},
    which we omit here for succintness.
    We do so also for class constraint imposed by generic algebraic operations like @{text "+"}, @{text "*"} etc.
  }.
  To avoid notational clutter, we omit the coercion functions between @{typ "'b \<Rightarrow>\<^sub>0 'a"} and @{typ "'b \<Rightarrow> 'a"}; for example, we use ordinary syntax for function application.
  Hence, @{text "power_product"} is isomorphic to @{typ "nat \<Rightarrow>\<^sub>0 nat"} and 
  a (multivariate) polynomial in turn is isomorphic to an almost everywhere
  zero mapping from power products to coefficients (@{typ "(nat \<Rightarrow>\<^sub>0 nat) \<Rightarrow>\<^sub>0 'a"}).
  Furthermore, the type @{typ "'b \<Rightarrow>\<^sub>0 'a"} is also the basis of several representation types in the next section.

  The advantages of this ``functional approach'' are already visible at this early stage of development:
  any combination of polynomials by operations is conceptionally clean.
  That is, polynomials are implicitly extended to any higher number of variables, as their exponents are mapped to zero already anyway.
  In the sense of \S\ref{goals}, we model the number of variables implicit.
  
  For example, we obtain the additive structure of polynomials @{typ "(nat \<Rightarrow>\<^sub>0 nat) \<Rightarrow>\<^sub>0 'a"} by instantiating @{typ "'b \<Rightarrow>\<^sub>0 'a"} pointwise.
  Clearly, the number of variables of a polynomial is irrelevant for these specifications.
  By our design choices, it does not show up, either.
  \begin{center}
    \begin{tabular}{@ {}l@ {\qquad}l@ {\qquad}l@ {}}
      @{text "0 = (\<lambda>b. 0)"}
      &
      @{text "1 = (\<lambda>b. 1)"}
      &
      \\
      @{text "(f + g) b = f b + g b"}
      &
      @{text "(- f) b = - f b"}
      &
      @{text "(f - g) b = f b - g b"}
    \end{tabular}
  \end{center}
  The definition of multiplication on @{typ "'b \<Rightarrow>\<^sub>0 'a"} is also similar to the canonical definition on polymonials.
  It delegates to addition on @{typ 'b} and multiplication on @{typ 'a}:  
  \begin{center}
    @{text "(f * g) b = (\<Sum>(a, a')\<in>{(a, a'). a + a'= b}. f a * g a')"}
  \end{center}
  Here, the definition of addition on @{typ "'b \<Rightarrow>\<^sub>0 'a"} plays a role, too:
  On polynomials (i.e., when @{typ "'b"} is instantiated by @{typ "nat \<Rightarrow>\<^sub>0 nat"}), the addition on @{typ "nat \<Rightarrow>\<^sub>0 nat"} turns into addition of exponents of variables, and the referenced multiplication on @{typ 'a} to multiplication of coefficients.  
  

*}

end
